package henshinerror.handlers;

import java.net.URL;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.henshin.interpreter.EGraph;
import org.eclipse.emf.henshin.interpreter.Engine;
import org.eclipse.emf.henshin.interpreter.Match;
import org.eclipse.emf.henshin.interpreter.RuleApplication;
import org.eclipse.emf.henshin.interpreter.impl.EGraphImpl;
import org.eclipse.emf.henshin.interpreter.impl.EngineImpl;
import org.eclipse.emf.henshin.interpreter.impl.LoggingApplicationMonitor;
import org.eclipse.emf.henshin.interpreter.impl.MatchImpl;
import org.eclipse.emf.henshin.interpreter.impl.RuleApplicationImpl;
import org.eclipse.emf.henshin.interpreter.util.InterpreterUtil;
import org.eclipse.emf.henshin.model.Rule;
import org.eclipse.emf.henshin.model.resource.HenshinResourceSet;
import org.eclipse.uml2.uml.UMLPackage;
import org.eclipse.uml2.uml.resource.UMLResource;
import org.osgi.framework.Bundle;

/**
 * Our sample handler extends AbstractHandler, an IHandler base class.
 * 
 * @see org.eclipse.core.commands.IHandler
 * @see org.eclipse.core.commands.AbstractHandler
 */
public class SampleHandler extends AbstractHandler {
	private HenshinResourceSet resourceSet;
	private EGraph graph;

	/**
	 * The constructor.
	 */
	public SampleHandler() {
	}

	/**
	 * the command has been executed, so extract extract the needed information
	 * from the application context.
	 */
	public Object execute(ExecutionEvent event)
			throws ExecutionException {
		load();
		testMethod();
		return null;
	}

	private Rule getRuleByTransformationModel(String rule, String file) {
		Bundle bundle = Platform.getBundle("HenshinError");
		Path path = new Path("resources/" + file);
		URL fileURL = FileLocator.find(bundle, path, null);
		return (Rule) resourceSet.getModule(org.eclipse.emf.common.util.URI.createURI(fileURL.toString()), false)
				.getUnit(rule);
	}

	public void load() {
		EPackage.Registry.INSTANCE.put(UMLPackage.eNS_URI, UMLPackage.eINSTANCE);
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put(UMLResource.FILE_EXTENSION,
				UMLResource.Factory.INSTANCE);

		// Create a resource set with a base directory:
		resourceSet = new HenshinResourceSet(null);

		resourceSet.getPackageRegistry().put(UMLPackage.eNS_URI, UMLPackage.eINSTANCE);
		resourceSet.registerXMIResourceFactories("uml");
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put(UMLResource.FILE_EXTENSION,
				UMLResource.Factory.INSTANCE);

		// Load the model into an EGraph:
		Bundle bundle = Platform.getBundle("HenshinError");
		
		//---------------------------------------------------------------
		
		
		// In testmodel.uml, the model refers to the same **file** as the Henshin module. Everything is OK.
		Path path = new Path("resources/testmodel.uml");
		
		//The alternate model refers to the profile copy in the other directory. Henshin does not execute the rule.
		
		//Path path = new Path("resources/testmodel_error.uml");
		
		
		
		
		
		//---------------------------------------------------------------
		URL fileURL = FileLocator.find(bundle, path, null);
		Resource resource = resourceSet.getResource(org.eclipse.emf.common.util.URI.createURI(fileURL.toString()),
				true);

		graph = new EGraphImpl(resource);

	}

	private void testMethod() {
		Engine engine = new EngineImpl();

		Rule rule = (Rule) this.getRuleByTransformationModel("testrule", "stereotest.henshin");
		Match partialMatch = new MatchImpl(rule);

		RuleApplication ruleApp = new RuleApplicationImpl(new EngineImpl(), graph, partialMatch.getRule(),
				partialMatch);
		for (Match m : InterpreterUtil.findAllMatches(engine,
				getRuleByTransformationModel("testrule", "stereotest.henshin"), graph, null)) {
			System.out.println(m);
		}
		if (!ruleApp.execute(new LoggingApplicationMonitor())) {
			System.out.println("Rule is not executed :-(");
		} else
			System.out.println("Everything allright :-)");
	}
}
